'From Squeak3.11alpha of 19 December 2009 [latest update: #8507] on 22 December 2009 at 10:09:03 am'!

!AdditionalMethodState methodsFor: 'accessing' stamp: 'ar 12/22/2009 07:54'!
methodHome
	"The behavior (trait/class) this method was originally defined in. 
	The methodClass in AdditionalMethodState but subclasses 
	(TraitMethodState) may know differently"
	^method methodClass! !

!AdditionalMethodState methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/22/2009 08:15'!
originalTraitMethod
	"The original method from the trait.
	Only available in TraitMethodState."
	^nil! !


!Association methodsFor: 'comparing' stamp: 'ar 12/21/2009 21:44'!
analogousCodeTo: anObject 
	"For comparing CompiledMethods"
	^self class == anObject class
		  and: [key == anObject key
		  and: [value = anObject value]]! !


!Behavior methodsFor: 'adding/removing methods' stamp: 'ar 12/20/2009 12:58'!
basicRemoveSelector: selector 
	"Assuming that the argument, selector (a Symbol), is a message selector 
	in my method dictionary, remove it and its method. Returns the old method
	if found, nil otherwise."

	| oldMethod |
	oldMethod := self methodDict at: selector ifAbsent: [^ nil].
	self methodDict removeKey: selector.

	"Now flush Squeak's method cache, either by selector or by method"
	oldMethod flushCache.
	selector flushCache.
	^oldMethod! !

!Behavior methodsFor: 'adding/removing methods' stamp: 'ar 12/20/2009 14:27'!
removeSelector: aSelector 
	"Assuming that the argument, selector (a Symbol), is a message selector 
	in my method dictionary, remove it and its method.
	
	If the method to remove will be replaced by a method from my trait composition,
	the current method does not have to be removed because we mark it as non-local.
	If it is not identical to the actual method from the trait it will be replaced automatically
	by #noteChangedSelectors:.
	
	This is useful to avoid bootstrapping problems when moving methods to a trait
	(e.g., from TPureBehavior to TMethodDictionaryBehavior). Manual moving (implementing
	the method in the trait and then remove it from the class) does not work if the methods
	themselves are used for this process (such as compiledMethodAt:, includesLocalSelector: or
	addTraitSelector:withMethod:)"

	| changeFromLocalToTraitMethod |
	(self traitComposition isKindOf: TraitComposition) 
		ifFalse:[^self basicRemoveSelector: aSelector].

	changeFromLocalToTraitMethod := (self includesLocalSelector: aSelector)
		and: [self hasTraitComposition]
		and: [self traitComposition includesMethod: aSelector].

	changeFromLocalToTraitMethod
		ifFalse: [self basicRemoveSelector: aSelector]
		ifTrue: [self ensureLocalSelectors].
	self deregisterLocalSelector: aSelector.
	self noteChangedSelectors: (Array with: aSelector)
	
! !

!Behavior methodsFor: 'traits' stamp: 'al 7/30/2004 21:48'!
setTraitComposition: aTraitComposition
	| oldComposition |
	(self hasTraitComposition not and: [aTraitComposition isEmpty]) ifTrue: [^self].
	aTraitComposition assertValidUser: self.

	oldComposition := self traitComposition.
	self traitComposition: aTraitComposition.
	self applyChangesOfNewTraitCompositionReplacing: oldComposition.
	
	oldComposition traits do: [:each | each removeUser: self].
	aTraitComposition traits do: [:each | each addUser: self]! !


!ClassDescription methodsFor: 'accessing method dictionary' stamp: 'ar 12/22/2009 09:50'!
removeSelector: selector 
	"Remove the message whose selector is given from the method 
	dictionary of the receiver, if it is there. Answer nil otherwise."

	| priorMethod priorProtocol | 
	priorMethod := self compiledMethodAt: selector ifAbsent: [^ nil].
	priorProtocol := self whichCategoryIncludesSelector: selector.
	(self traitComposition isKindOf: NanoTraitComposition) ifTrue:[
		SystemChangeNotifier uniqueInstance doSilently: [
			self organization removeElement: selector].
	].
	super removeSelector: selector.
	SystemChangeNotifier uniqueInstance 
		doSilently: [self updateOrganizationSelector: selector oldCategory: priorProtocol newCategory: nil].
	SystemChangeNotifier uniqueInstance 
			methodRemoved: priorMethod selector: selector inProtocol: priorProtocol class: self.! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'ar 12/20/2009 13:32'!
fileOutCategory: aSymbol on: aFileStream moveSource: moveSource toFile: fileIndex 
	"File a description of the receiver's category, aString, onto aFileStream. If 
	moveSource, is true, then set the method source pointer to the new file position.
	Note when this method is called with moveSource=true, it is condensing the
	.sources file, and should only write one preamble per method category."

	| selectors |

	aFileStream cr.
	selectors := (aSymbol asString = ClassOrganizer allCategory)
				ifTrue: [ self organization allMethodSelectors ]
				ifFalse: [ self organization listAtCategoryNamed: aSymbol ].

	selectors := selectors select: [:each | (self includesLocalSelector: each)].
	
	"Overridden to preserve author stamps in sources file regardless"
	selectors do: [:sel |
		self printMethodChunk: sel 
			withPreamble: true
			on: aFileStream 
			moveSource: moveSource 
			toFile: fileIndex].
	^ self! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'ar 12/20/2009 14:24'!
fileOutInitializerOn: aStream
	"If the receiver has initialization, file it out."! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'ar 12/20/2009 13:10'!
fileOutOn: aFileStream moveSource: moveSource toFile: fileIndex initializing: aBool
	^self fileOutOn: aFileStream moveSource: moveSource toFile: fileIndex! !

!ClassDescription methodsFor: 'organization updating' stamp: 'ar 12/20/2009 14:28'!
updateOrganizationSelector: aSymbol oldCategory: oldCategoryOrNil newCategory: newCategoryOrNil
	| currentCategory effectiveCategory sel changedCategories composition |
	changedCategories := IdentitySet new.
	composition := self hasTraitComposition
		ifTrue: [self traitComposition]
		ifFalse: [TraitComposition new].
	(composition isKindOf: TraitComposition) ifFalse:[^self].
	(composition methodDescriptionsForSelector: aSymbol) do: [:each |
		sel := each selector.
		(self includesLocalSelector: sel) ifFalse: [
			currentCategory := self organization categoryOfElement: sel.
			effectiveCategory := each effectiveMethodCategoryCurrent: currentCategory new: newCategoryOrNil.
			effectiveCategory isNil ifTrue: [
				currentCategory ifNotNil: [changedCategories add: currentCategory].
				self organization removeElement: sel.
			] ifFalse: [
				((currentCategory isNil or: [currentCategory == ClassOrganizer ambiguous or: [currentCategory == oldCategoryOrNil]]) and: [currentCategory ~~ effectiveCategory]) ifTrue: [
					currentCategory ifNotNil: [changedCategories add: currentCategory].
					self organization 
						classify: sel 
						under: effectiveCategory
						suppressIfDefault: false]]]].
	^ changedCategories! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/21/2009 15:16'!
allTraits
	"Answer all the traits that are used by myself without their transformations"
	^self traitComposition allTraits.! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 00:06'!
basicRemoveSelector: aSelector
	"Remove the message whose selector is given from the method 
	dictionary of the receiver, if it is there. Update the trait composition."
	| oldMethod |
	oldMethod := super basicRemoveSelector: aSelector.
	oldMethod ifNotNil:[self updateTraits].
	^oldMethod! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 12:52'!
classify: selector under: heading from: category trait: aTrait
	"Update the organization for a trait. the dumb, unoptimized version"
	self updateTraits.! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 12:59'!
hasTraitComposition
	^self traitComposition notEmpty! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 13:29'!
includesLocalSelector: selector
	^(self traitComposition isKindOf: NanoTraitComposition)
		ifTrue:[(self compiledMethodAt: selector ifAbsent:[^false]) methodHome == self]
		ifFalse:[super includesLocalSelector: selector].! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 02:36'!
includesTrait: aTrait
	^self traitComposition includesTrait: aTrait! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 13:38'!
installTraitsFrom: aTraitComposition
	"Install the traits from the given composition"
	| allTraits methods oldMethod removals oldCategories |
	(aTraitComposition isKindOf: NanoTraitComposition) 
		ifFalse:[self error: 'Invalid composition'].
	(self traitComposition isEmpty and: [aTraitComposition isEmpty]) ifTrue: [^self].

	"Check for cycles"
	allTraits := aTraitComposition gather: [:t | t allTraits copyWith: t].
	(allTraits includes: self) ifTrue:[^self error: 'Cyclic trait definition detected'].

	"XXXX: addUser/removeUser should be part of setter, but subclass 
	override prevents it until we've got rid of Traits mess."
	self traitComposition removeTraitUser: self.
	self traitComposition: aTraitComposition.
	aTraitComposition addTraitUser: self.

	"Assemble the methods in a new dictionary first.
	Uses a Dictionary instead of a MethodDictionary for speed (MDs grow by #become:)"
	methods := Dictionary new.

	"Stick in the local methods first, since this avoids generating conflict methods unnecessarily"
	self selectorsAndMethodsDo:[:sel :newMethod|
		(self isLocalMethod: newMethod)
			ifTrue:[methods at: sel put:newMethod]].

	"Now assemble the traits methods"
	aTraitComposition do:[:trait|
		trait selectorsAndMethodsDo:[:sel :newMethod|
			oldMethod := methods at: sel ifAbsentPut:[newMethod].
			newMethod == oldMethod ifFalse:["a conflict"
				(self isLocalMethod: oldMethod) ifFalse:[
					methods at: sel put: (self resolveTraitsConflict: sel from: oldMethod to: newMethod).
				].
			].
		].
	].

	"Apply the changes. We first add the new or changed methods."
	oldCategories := Set new.
	methods keysAndValuesDo:[:sel :newMethod|
		oldMethod := self compiledMethodAt: sel ifAbsent:[nil].
		oldMethod == newMethod ifFalse:[
			self traitAddSelector: sel withMethod: newMethod.
			(self organization categoryOfElement: sel) ifNotNil:[:cat| oldCategories add: cat].
			self organization classify: sel under: 
				(newMethod methodHome organization categoryOfElement: newMethod selector).
		]].

	"Now remove the old or obsoleted ones"
	removals := OrderedCollection new.
	self selectorsDo:[:sel| (methods includesKey: sel) ifFalse:[removals add: sel]].
	removals do:[:sel| self traitRemoveSelector: sel].

	"Clean out empty categories"
	oldCategories do:[:cat|
		(self organization isEmptyCategoryNamed: cat)
			ifTrue:[self organization removeCategory: cat]].

	self isMeta ifFalse:[self class updateTraitsFrom: aTraitComposition].! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 13:31'!
isAliasSelector: aSymbol
	"Return true if the selector aSymbol is an alias defined
	in my or in another composition somewhere deeper in 
	the tree of traits compositions."

	^(self includesLocalSelector: aSymbol) not
		and: [self hasTraitComposition]
		and: [self traitComposition isAliasSelector: aSymbol]! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 13:31'!
isLocalAliasSelector: aSymbol
	"Return true if the selector aSymbol is an alias defined
	in my or in another composition somewhere deeper in 
	the tree of traits compositions."

	^(self includesLocalSelector: aSymbol) not
		and: [self hasTraitComposition]
		and: [self traitComposition isLocalAliasSelector: aSymbol]! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 11/28/2007 20:51'!
isLocalMethod: aCompiledMethod
	"Answer true if the method is a local method, e.g., defined in the receiver instead of a trait."
	^aCompiledMethod methodHome == self! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 13:30'!
localSelectors
	^(self traitComposition isKindOf: NanoTraitComposition)
		ifTrue:[self selectors select:[:sel| self includesLocalSelector: sel]]
		ifFalse:[super localSelectors]! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/19/2009 21:14'!
replaceSelector: originalSelector withAlias: aliasSelector in: source
	"replaces originalSelector with aliasSelector in in given source code"
	| oldKeywords newKeywords args selectorWithArgs s |
	oldKeywords := originalSelector keywords.
	newKeywords := aliasSelector keywords.
	oldKeywords size = newKeywords size ifFalse:[self error: 'Keyword mismatch'].
	args := (self parserClass new parseArgsAndTemps: source asString notifying: nil) 
				copyFrom: 1 to: originalSelector numArgs.
	selectorWithArgs := String streamContents: [:stream |
		newKeywords keysAndValuesDo: [:index :keyword |
			stream nextPutAll: keyword.
			stream space.
			args size >= index ifTrue: [
				stream nextPutAll: (args at: index); space]]].
	s := source asString readStream.
	oldKeywords do: [ :each | s match: each ].
	args isEmpty ifFalse: [ s match: args last ].
	^selectorWithArgs withBlanksTrimmed asText , s upToEnd
! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/21/2009 19:53'!
resolveTraitsConflict: aSelector from: oldMethod to: newMethod
	"Resolve a traits conflict. Rules:
		- If one method is required the other one wins
		- Otherwise we compile a traits conflict
	"
	| marker selector |
	oldMethod methodHome == newMethod methodHome ifTrue:[^oldMethod].
	marker := oldMethod markerOrNil.
	(#(requirement explicitRequirement subclassResponsibility shouldNotImplement) includes: marker)
		ifTrue:[^newMethod].
	marker := newMethod markerOrNil.
	(#(requirement explicitRequirement subclassResponsibility shouldNotImplement) includes: marker)
		ifTrue:[^oldMethod].
	"Create a conflict marker"
	selector := #(conflict conflict: conflict:with: conflict:with:with: conflict:with:with:with:
	conflict:with:with:with:with: conflict:with:with:with:with:with: conflict:with:with:with:with:with:with:
	conflict:with:with:with:with:with:with:with:) at: oldMethod numArgs+1.
	^NanoTraitDescription class compiledMethodAt: selector.! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/21/2009 20:06'!
setTraitComposition: aTraitComposition
	"OBSOLETE. Use Class uses: aTraitComposition instead."
	(aTraitComposition isKindOf: NanoTraitComposition)
		ifTrue:[^self uses: aTraitComposition].
	(aTraitComposition isKindOf: TraitComposition)
		ifTrue:[^super setTraitComposition: aTraitComposition].
	"Unspecified. Check for prevailing traitOverride"
	Trait traitOverride == NanoTrait 
		ifTrue:[^self uses: aTraitComposition]
		ifFalse:[^super setTraitComposition: aTraitComposition].! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/21/2009 20:06'!
setTraitCompositionFrom: aTraitComposition
	"OBSOLETE. Use Class uses: aTraitComposition instead."
	(aTraitComposition isKindOf: NanoTraitComposition)
		ifTrue:[^self uses: aTraitComposition].
	(aTraitComposition isKindOf: TraitComposition)
		ifTrue:[^super setTraitCompositionFrom: aTraitComposition].
	"Unspecified. Check for prevailing traitOverride"
	Trait traitOverride == NanoTrait 
		ifTrue:[^self uses: aTraitComposition]
		ifFalse:[^super setTraitCompositionFrom: aTraitComposition].! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/22/2009 10:02'!
traitAddSelector: selector withMethod: traitMethod
	"Add a method inherited from a trait. 
	Recompiles to avoid sharing and implement aliasing."
	| oldMethod source methodNode newMethod originalSelector |
	oldMethod := self compiledMethodAt: selector ifAbsent:[nil].
	oldMethod ifNotNil:[
		"The following is an important optimization as it prevents exponential
		growth in recompilation. If T1 is used by T2 and T2 by T3 then (without
		this optimization) any change in T1 would cause all methods in T2 to be
		recompiled and each recompilation of a method in T2 would cause T3
		to be fully recompiled. The test eliminates all such situations."
		(oldMethod sameTraitCodeAs: traitMethod) ifTrue:[^oldMethod].
	].
	originalSelector := traitMethod selector.
	source := traitMethod methodClass sourceCodeAt: originalSelector.
	originalSelector == selector ifFalse:[
		"Replace source selectors for aliases"
		source := self replaceSelector: originalSelector withAlias: selector in: source.
	].
	methodNode := self compilerClass new 
		compile: source in: self classified: nil notifying: nil ifFail:[^nil].
	newMethod := methodNode generate: self defaultMethodTrailer.
	newMethod putSource: source fromParseNode: methodNode inFile: 2
		withPreamble: [:f | f cr; nextPut: $!!; nextChunkPut: 'Trait method'; cr].
	newMethod originalTraitMethod: traitMethod.
	^super addSelectorSilently: selector withMethod: newMethod.! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/22/2009 08:33'!
traitComposition
	"Answer my trait composition"
	^nil! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/22/2009 08:33'!
traitComposition: aTraitComposition
	"Install my traits"
	^self subclassResponsibility! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 03:13'!
traitRemoveSelector: selector 
	"Remove the message whose selector is given from the method 
	dictionary of the receiver, if it is there. Answer nil otherwise."
	| priorMethod priorProtocol | 
	priorMethod := self compiledMethodAt: selector ifAbsent: [^ nil].
	priorProtocol := self whichCategoryIncludesSelector: selector.
	SystemChangeNotifier uniqueInstance doSilently: [
		self organization removeElement: selector].
	super basicRemoveSelector: selector.
	SystemChangeNotifier uniqueInstance 
			methodRemoved: priorMethod selector: selector inProtocol: priorProtocol class: self.
	(self organization isEmptyCategoryNamed: priorProtocol)
		ifTrue:[self organization removeCategory: priorProtocol].
! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/21/2009 15:13'!
traits
	"Answer an array of my traits"
	^self traitComposition asArray collect:[:composed| composed trait]! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 13:19'!
updateTraits
	"Recompute my local traits composition"
	(self traitComposition isKindOf: NanoTraitComposition)
		ifTrue:[self installTraitsFrom: self traitComposition].
! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/19/2009 23:43'!
users
	^#()! !

!ClassDescription methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/20/2009 12:38'!
uses: aTraitComposition
	| newTraits |
	newTraits := (aTraitComposition isKindOf: NanoTrait orOf: NanoTraitTransformation) 
		ifTrue:[NanoTraitComposition with: aTraitComposition]
		ifFalse:[(aTraitComposition isKindOf: SequenceableCollection)
					ifTrue:[NanoTraitComposition withAll: aTraitComposition asArray]
					ifFalse:[self error: 'Invalid traits specification']].
	self installTraitsFrom: newTraits.
! !


!Class methodsFor: 'accessing' stamp: 'ar 12/21/2009 20:24'!
traitComposition
	traitComposition ifNil: [traitComposition := Trait newTraitComposition].
	^traitComposition! !


!ChangeSet class methodsFor: 'fileIn/Out' stamp: 'ar 12/20/2009 13:04'!
doWeFileOut: aClass given: aSet cache: cache
	| aClassAllSuperclasses aClassSoleInstanceAllSuperclasses |
	aClassAllSuperclasses := cache at: aClass ifAbsentPut:[
		aClass allSuperclasses asArray, aClass allTraits.
	].
	(aSet includesAnyOf: aClassAllSuperclasses) ifTrue: [^false].
	aClass isMeta ifFalse: [^true].
	(aSet includes: aClass soleInstance) ifTrue: [^false].
	aClassSoleInstanceAllSuperclasses := cache at: aClass soleInstance
		ifAbsentPut: [aClass soleInstance allSuperclasses asArray, aClass soleInstance allTraits].
	(aSet includesAnyOf: aClassSoleInstanceAllSuperclasses) ifTrue: [^false].
	^true! !


!Metaclass methodsFor: 'accessing' stamp: 'ar 12/21/2009 20:25'!
traitComposition
	traitComposition ifNil: [traitComposition := Trait newTraitComposition].
	^traitComposition! !

!Metaclass methodsFor: 'fileIn/Out' stamp: 'ar 12/20/2009 14:10'!
definition
	"Refer to the comment in ClassDescription|definition."

	^ String streamContents:[:strm |
		strm print: self.
		self traitComposition isEmpty ifFalse:[
			strm crtab; nextPutAll: 'uses: '; nextPutAll: self traitComposition asString.
		].
		strm
			crtab;
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString]! !

!Metaclass methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/21/2009 15:13'!
updateTraitsFrom: instanceTraits
	"Update me from the given instance traits"
	| map newTraits trait |
	((instanceTraits isKindOf: NanoTraitComposition) or:[instanceTraits isEmpty]) 
		ifFalse:[self error: 'Invalid trait'].
	map := Dictionary new.
	self traitComposition isEmpty ifFalse:[
		self traitComposition do:[:composed| map at: composed trait put: composed].
	].
	instanceTraits isEmpty ifFalse:[
		newTraits := (instanceTraits collect:[:composed|
			trait := composed trait classTrait.
			map at: trait ifAbsent:[trait]] 
		).
	].
	self traitComposition isEmpty ifFalse:[ 
		newTraits := newTraits, (self traitComposition select:[:comp| comp trait isBaseTrait]).
	].
	self installTraitsFrom: newTraits! !


!SequenceableCollection methodsFor: '*Traits' stamp: 'ar 12/21/2009 20:25'!
asTraitComposition
	"For convenience the composition {T1. T2 ...} is the same as T1 + T2 + ..."
	^self isEmpty
		ifFalse: [
			self size = 1
				ifTrue: [self first asTraitComposition]
				ifFalse: [
					self copyWithoutFirst 
						inject: self first
						into: [:left :right | left + right]]]
		ifTrue: [Trait newTraitComposition]! !


!CompiledMethod methodsFor: 'accessing' stamp: 'ar 12/22/2009 07:46'!
methodHome
	"The behavior (trait/class) this method was originally defined in. 
	Can be different from methodClass if the method was recompiled."
	^self properties methodHome! !

!CompiledMethod methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/22/2009 08:23'!
originalTraitMethod
	"Remember the original trait method for the receiver."
	^self properties originalTraitMethod! !

!CompiledMethod methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/22/2009 07:57'!
originalTraitMethod: aCompiledMethod
	"Remember the original trait method for the receiver."
	| methodState |
	methodState := NanoTraitMethodState newFrom: self properties.
	methodState originalTraitMethod: aCompiledMethod.
	self penultimateLiteral:  methodState.! !

!CompiledMethod methodsFor: '*NanoTraits-Extensions' stamp: 'ar 12/22/2009 10:04'!
sameTraitCodeAs: method
	"Answer whether the receiver implements the same code as the 
	argument, method. Does not look at properties/pragmas since they
	do not affect the resulting code."
	| numLits |
	(method isKindOf: CompiledMethod) ifFalse: [^false].
	self methodHome == method methodHome ifFalse:[^false].
	(self properties analogousCodeTo: method properties) ifFalse:[^false].
	self size = method size ifFalse: [^false].
	self header = method header ifFalse: [^false].
	self initialPC to: self endPC do:[:i | (self at: i) = (method at: i) ifFalse: [^false]].
	(numLits := self numLiterals) ~= method numLiterals ifTrue: [^false].
	1 to: numLits-2 do:[:i| | lit1 lit2 |
		lit1 := self literalAt: i.
		lit2 := method literalAt: i.
		lit1 = lit2 ifFalse:[
			(i = 1 and: [#(117 120) includes: self primitive]) ifTrue: [
				lit1 isArray ifTrue:[
					(lit2 isArray and: [lit1 allButLast = lit2 allButLast]) ifFalse:[^false]
				] ifFalse: "ExternalLibraryFunction"
					[(lit1 analogousCodeTo: lit2) ifFalse:[^false]].
			] ifFalse:[
				lit1 isFloat 
					ifTrue:[(lit1 closeTo: lit2) ifFalse: [^false]]
					ifFalse:["any other discrepancy is a failure"^ false]]]].
	^true! !


!SystemDictionary methodsFor: 'class and trait names' stamp: 'ar 12/20/2009 14:21'!
traitNames
	"Answer a SortedCollection of all traits (not including class-traits) names."
	| names |
	names := OrderedCollection new.
	self do: 
		[:cl | (cl isInMemory
			and: [(cl isTrait)
			and: [(cl name beginsWith: 'AnObsolete') not]])
				ifTrue: [names add: cl name]].
	^ names! !


!SystemNavigation methodsFor: 'query' stamp: 'ar 12/21/2009 19:59'!
allBehaviorsDo: aBlock 
	"Evaluate the argument, aBlock, for each kind of Behavior in the system 
	(that is, Object and its subclasses and Traits).
	ar 7/15/1999: The code below will not enumerate any obsolete or anonymous
	behaviors for which the following should be executed:

		Smalltalk allObjectsDo:[:obj| obj isBehavior ifTrue:[aBlock value: obj]].

	but what follows is way faster than enumerating all objects."

	aBlock value: ProtoObject.
	ProtoObject allSubclassesDoGently: aBlock.		"don't bring in ImageSegments"

	"Classes outside the ProtoObject hierarchy"
	Class subclassesDo: [:aClass |
		(aClass == ProtoObject class
			or: [aClass isInMemory not
			or: [aClass isMeta not]]) ifFalse:
			["Enumerate the non-meta class and its subclasses"
			aBlock value: aClass soleInstance.
			aClass soleInstance allSubclassesDoGently: aBlock]].
	
	Trait allInstances , ClassTrait allInstances do: [:trait |
		aBlock value: trait].

	NanoClassTrait allInstances do: [:metaTrait|
		aBlock value: metaTrait instanceSide.
		aBlock value: metaTrait.
	].
	
! !


!Trait class methodsFor: 'instance creation' stamp: 'ar 12/21/2009 19:01'!
named: aSymbol uses: aTraitCompositionOrCollection category: aString env: anEnvironment
	| trait oldTrait systemCategory |
	"Hook for alternative implementations"
	TraitOverride ifNotNil:[^TraitOverride named: aSymbol uses: aTraitCompositionOrCollection category: aString env: anEnvironment].
	systemCategory := aString asSymbol.
	trait := anEnvironment
		at: aSymbol
		ifAbsent: [nil].
	oldTrait := trait copy.
	trait := trait ifNil: [super new].
	
	(trait isKindOf: Trait) ifFalse: [
		^self error: trait name , ' is not a Trait'].
	trait
		setName: aSymbol
		andRegisterInCategory: systemCategory
		environment: anEnvironment.
		
	trait setTraitComposition: aTraitCompositionOrCollection asTraitComposition.
	
	"... notify interested clients ..."
	oldTrait isNil ifTrue: [
		SystemChangeNotifier uniqueInstance classAdded: trait inCategory: systemCategory.
		^ trait].
	SystemChangeNotifier uniqueInstance traitDefinitionChangedFrom: oldTrait to: trait.
	systemCategory ~= oldTrait category 
		ifTrue: [SystemChangeNotifier uniqueInstance class: trait recategorizedFrom: oldTrait category to: systemCategory].
		
	^ trait! !

!Trait class methodsFor: 'accessing' stamp: 'ar 12/21/2009 20:24'!
newTraitComposition
	"Creates a new TraitComposition"
	^TraitOverride 
		ifNil:[TraitComposition new]
		ifNotNil:[TraitOverride newTraitComposition].! !

!Trait class methodsFor: 'accessing' stamp: 'ar 12/21/2009 19:01'!
traitOverride
	"Current override for Trait"
	^TraitOverride! !

!Trait class methodsFor: 'accessing' stamp: 'ar 12/21/2009 19:02'!
traitOverride: aTraitClassOrNil
	"Current override for Trait"
	"Trait traitOverride: NanoTrait"
	"Trait traitOverride: nil"
	TraitOverride := aTraitClassOrNil.
! !


!TraitComposition methodsFor: 'accessing' stamp: 'ar 12/20/2009 13:51'!
removeTraitUser: aClass
	self traits do:[:each| each removeUser: aClass].! !

!TraitComposition methodsFor: 'converting' stamp: 'ar 12/20/2009 14:09'!
asArray
	^self transformations asArray! !

CompiledMethod removeSelector: #methodHome:!
CompiledMethod removeSelector: #sameCodeAs:!

!CompiledMethod reorganize!
('accessing' clearFlag defaultSelector dragLabel endPC flag flushCache frameSize initialPC methodClass methodClass: methodClassAssociation methodHome methodReference numArgs numLiterals numTemps primitive properties properties: returnField scanner searchForClass searchForSelector selector selector: trailer)
('closures' allEmbeddedBlockMethods containsBlockClosures embeddedBlockMethods isBlockMethod isBlockMethod: isClosureCompiled: method remoteReturns searchImageForHomeMethod)
('comparing' equivalentTo: =)
('decompiling' compilerClass decompile decompilerClass decompileWithTemps methodNode methodNodeFormattedAndDecorated: parserClass)
('evaluating' valueWithReceiver:arguments:)
('file in/out' objectForDataStream: readDataFrom:size: storeDataOn: veryDeepCopyWith: zapSourcePointer)
('initialize-release' copyWithTrailerBytes: needsFrameSize:)
('literals' allLiterals hasLiteral: hasLiteralSuchThat: hasLiteralThorough: hasLiteralThorough:from: header headerDescription indexOfLiteral: literalAt: literalAt:put: literalStrings literals literalsDo: objectAt: objectAt:put: refersToLiteral: sendsSelector:)
('printing' abstractSymbolic dateMethodLastSubmitted decompileString longPrintOn: longPrintOn:indent: longPrintRelativeOn:indent: primitiveErrorVariableName printOn: printOnStream: printPrimitiveOn: storeLiteralsOn:forClass: storeOn: symbolic symbolicLinesDo: timeStamp)
('scanning' hasInstVarRef messages messagesDo: messagesSequence readsField: readsRef: scanFor: scanLongLoad: scanLongStore: scanVeryLongLoad:offset: scanVeryLongStore:offset: sendsToSuper writesField: writesRef:)
('source code management' checkOKToAdd:at: copyWithTempNames: copyWithTempsFromMethodNode: fileIndex filePosition getPreambleFrom:at: getSource getSourceFor:in: getSourceFromFile holdsTempNames linesOfCode putSource:fromParseNode:class:category:inFile:priorMethod: putSource:fromParseNode:class:category:withStamp:inFile:priorMethod: putSource:fromParseNode:inFile:withPreamble: qCompress: qDecompressFrom: setMySourcePointer: setSourcePointer: setSourcePosition:inFile: sourceClass sourceFileStream sourcePointer sourceSelector tempNamesString)
('testing' hasNewPropertyFormat hasReportableSlip isAbstract isBlueBookCompiled isClosureCompiled isCompiledMethod isConflict isDisabled isDisabled: isExplicitlyRequired isExplicitlyRequired: isImplicitlyRequired isImplicitlyRequired: isInstalled isProvided isProvided: isQuick isRequired isRequired: isReturnField isReturnSelf isReturnSpecial isSubclassResponsibility isSubclassResponsibility: usesClosureBytecodes)
('*Tools-Inspector' inspectorClass)
('private' markerOrNil penultimateLiteral penultimateLiteral: replace:with:in:)
('*39Deprecated')
('debugger support' abstractPCForConcretePC: blockExtentsInto:from:to:scanner:numberer: blockExtentsToTempsMap debuggerMap hasBreakpoint mapFromBlockKeys:toSchematicTemps: pcPreviousTo: startpcsToBlockExtents tempsSubSequenceFrom:)
('accessing-pragmas & properties' pragmaAt: pragmas propertyKeysAndValuesDo: propertyValueAt: propertyValueAt:ifAbsent: propertyValueAt:put: removeProperty: removeProperty:ifAbsent:)
('*Morphic-Explorer' explorerContents)
('*NanoTraits-Extensions' originalTraitMethod originalTraitMethod: sameTraitCodeAs:)
!


!ClassDescription reorganize!
('accessing' classVersion version)
('accessing class hierarchy' classesThatImplementAllOf: commentInventory printSubclassesOn:level: removeUninstantiatedSubclassesSilently subclasses subclassesDo:)
('accessing comment' classCommentBlank comment comment: comment:stamp: hasComment)
('accessing method dictionary' addAndClassifySelector:withMethod:inProtocol:notifying: addSelector:withMethod:notifying: addSelectorSilently:withMethod: allMethodCategoriesIntegratedThrough: allMethodsInCategory: induceMDFault isUniClass noteAddedSelector:meta: recoverFromMDFault recoverFromMDFaultWithTrace removeCategory: removeSelector:)
('accessing parallel hierarchy' classSide instanceSide isClassSide isInstanceSide isMeta theMetaClass theNonMetaClass)
('compiling' acceptsLoggingOfCompilation compile:classified: compile:classified:notifying: compile:classified:withStamp:notifying: compile:classified:withStamp:notifying:logSource: compile:notifying: compileSilently:classified: compileSilently:classified:notifying: doneCompiling instVarNamesAndOffsetsDo: moveInstVarNamed:to:after: noteCompilationOf:meta: reformatAll reformatMethodAt: wantsChangeSetLogging wantsRecompilationProgressReported)
('copying' copy:from: copy:from:classified: copyAll:from: copyAll:from:classified: copyAllCategoriesFrom: copyAllCategoriesUnobtrusivelyFrom: copyAllUnobtrusively:from:classified: copyCategory:from: copyCategory:from:classified: copyCategoryUnobtrusively:from: copyMethodDictionaryFrom: copyUnobtrusively:from:classified: copyUnobtrusivelyCategory:from:classified:)
('fileIn/Out' classComment: classComment:stamp: commentFollows commentStamp: commentStamp:prior: definition fileOutCategory: fileOutCategory:asHtml: fileOutCategory:on:moveSource:toFile: fileOutChangedMessages:on: fileOutChangedMessages:on:moveSource:toFile: fileOutChangedMessagesHistorically:on:moveSource:toFile: fileOutInitializerOn: fileOutMethod: fileOutMethod:asHtml: fileOutOn: fileOutOn:moveSource:toFile: fileOutOn:moveSource:toFile:initializing: fileOutOrganizationOn: methods methodsFor: methodsFor:priorSource:inFile: methodsFor:stamp: methodsFor:stamp:prior: moveChangesTo: moveChangesWithVersionsTo: printCategoryChunk:on: printCategoryChunk:on:priorMethod: printCategoryChunk:on:withStamp:priorMethod: printCategoryChunk:withStamp:on: printMethodChunk:withPreamble:on:moveSource:toFile: printMethodChunkHistorically:on:moveSource:toFile: putClassCommentToCondensedChangesFile:)
('initialize-release' forgetDoIts obsolete superclass:methodDictionary:format: updateInstances:from:isMeta: updateInstancesFrom:)
('instance variables' addInstVarName: allInstVarNamesEverywhere checkForInstVarsOK: chooseClassVarName chooseInstVarAlphabeticallyThenDo: chooseInstVarThenDo: classThatDefinesClassVariable: classThatDefinesInstanceVariable: forceNewFrom: instVarIndexFor:ifAbsent: instVarNameForIndex: instVarNames removeInstVarName: renameInstVar:to: renameSilentlyInstVar:to: replaceSilently:to:)
('organization' methodReferencesInCategory: organization organization: reorganize whichCategoryIncludesSelector: zapOrganization)
('organization updating' applyChangesOfNewTraitCompositionReplacing: noteRecategorizedSelector:from:to: noteRecategorizedSelectors:oldComposition: notifyOfRecategorizedSelector:from:to: updateOrganizationSelector:oldCategory:newCategory:)
('printing' classVariablesString instanceVariablesString printOn: printOnStream: printWithClosureAnalysisOn: sharedPoolsString storeOn:)
('*39Deprecated')
('*eToys-accessing method dictionary' namedTileScriptSelectors)
('*PlusTools')
('*system-support' allUnreferencedClassVariables)
('private' errorCategoryName instVarMappingFrom: linesOfCode logMethodSource:forMethodWithNode:inCategory:withStamp:notifying: newInstanceFrom:variable:size:map: setInstVarNames:)
('*NanoTraits')
('*NanoTraits-Extensions' allTraits basicRemoveSelector: classify:under:from:trait: hasTraitComposition includesLocalSelector: includesTrait: installTraitsFrom: isAliasSelector: isLocalAliasSelector: isLocalMethod: localSelectors replaceSelector:withAlias:in: resolveTraitsConflict:from:to: setTraitComposition: setTraitCompositionFrom: traitAddSelector:withMethod: traitComposition traitComposition: traitRemoveSelector: traits updateTraits users uses:)
!

