OrderedCollection subclass: #NanoTraitComposition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTraitComposition commentStamp: '<historical>' prior: 0!
A trait composition is a collection of Traits or TraitTransformations.!


!NanoTraitComposition methodsFor: 'converting' stamp: 'ar 12/21/2009 15:14'!
+ aTrait
	self traitsDo:[:t| (t == aTrait trait) ifTrue:[^NanoTraitCompositionException
			signal: 'Trait ' , aTrait trait asString, ' already in composition']].
	self addLast: aTrait.
	^self! !

!NanoTraitComposition methodsFor: 'converting' stamp: 'ar 11/18/2007 23:13'!
- anArray
	"the modifier operators #@ and #- bind stronger than +.
	Thus, #@ or #- sent to a sum will only affect the most right summand"
	
	self addLast: (self removeLast - anArray)! !

!NanoTraitComposition methodsFor: 'converting' stamp: 'ar 11/18/2007 23:12'!
@ anArrayOfAssociations
	"the modifier operators #@ and #- bind stronger than +.
	Thus, #@ or #- sent to a sum will only affect the most right summand"

	self addLast: (self removeLast @ anArrayOfAssociations)! !

!NanoTraitComposition methodsFor: 'converting' stamp: 'ar 12/4/2009 21:09'!
asTraitComposition
	^self! !

!NanoTraitComposition methodsFor: 'converting' stamp: 'ar 12/20/2009 02:38'!
printOn: aStream
	"Answer the trait composition string (used for class definitions)"
	aStream nextPutAll: self traitCompositionString.
! !


!NanoTraitComposition methodsFor: 'accessing' stamp: 'ar 12/20/2009 00:41'!
addTraitUser: aUser
	self do:[:each| each addTraitUser: aUser]! !

!NanoTraitComposition methodsFor: 'accessing' stamp: 'ar 12/20/2009 13:33'!
addUser: aUser
	^self addTraitUser: aUser! !

!NanoTraitComposition methodsFor: 'accessing' stamp: 'ar 12/21/2009 15:14'!
allTraits
	^self gather:[:each| each allTraits copyWith: each trait]! !

!NanoTraitComposition methodsFor: 'accessing' stamp: 'ar 12/20/2009 00:39'!
removeTraitUser: aUser
	self do:[:each| each removeTraitUser: aUser]! !

!NanoTraitComposition methodsFor: 'accessing' stamp: 'ar 12/20/2009 13:33'!
removeUser: aUser
	^self removeTraitUser: aUser! !

!NanoTraitComposition methodsFor: 'accessing' stamp: 'ar 5/11/2008 02:30'!
traits
	^Array streamContents:[:s| self traitsDo:[:t| s nextPut: t]]! !

!NanoTraitComposition methodsFor: 'accessing' stamp: 'ar 11/22/2007 06:21'!
traitsDo: aBlock
	^self do:[:each| each traitsDo: aBlock]! !


!NanoTraitComposition methodsFor: 'operations' stamp: 'ar 11/22/2007 06:20'!
copyTraitExpression
	"Copy all except the actual traits"
	^self collect:[:each| each copyTraitExpression].! !

!NanoTraitComposition methodsFor: 'operations' stamp: 'ar 5/11/2008 02:13'!
isAliasSelector: selector
	"enumerates all selectors and methods in a trait composition"
	^self anySatisfy:[:any| any isAliasSelector: selector]! !

!NanoTraitComposition methodsFor: 'operations' stamp: 'ar 12/19/2009 18:56'!
isLocalAliasSelector: selector
	"Return true if the selector aSymbol is an alias defined in the receiver."
	^self anySatisfy:[:any| any isTraitTransformation and:[any isLocalAliasSelector: selector]]! !

!NanoTraitComposition methodsFor: 'operations' stamp: 'ar 11/22/2007 06:12'!
selectorsAndMethodsDo: aBlock
	"enumerates all selectors and methods in a trait composition"
	self do:[:each| each selectorsAndMethodsDo: aBlock].! !

!NanoTraitComposition methodsFor: 'operations' stamp: 'ar 12/20/2009 13:17'!
traitCompositionString
	"Answer the trait composition string (used for class definitions)"
	self size = 0 ifTrue:[^'{}'].
	self  size = 1 ifTrue:[^self first asString].
	^String streamContents:[:s|
		self do:[:each| s nextPutAll: each asString] separatedBy:[s nextPutAll: ' + '].
	].! !


!NanoTraitComposition methodsFor: 'testing' stamp: 'ar 12/20/2009 02:35'!
includesTrait: aTrait
	^self anySatisfy:[:each| each includesTrait: aTrait]! !

!NanoTraitComposition methodsFor: 'testing' stamp: 'ar 11/28/2007 21:09'!
isTraitTransformation
	"Polymorphic with TraitTransformation"
	^false! !


!NanoTraitComposition methodsFor: 'compat' stamp: 'ar 12/20/2009 14:00'!
removeFromComposition: aTrait
	"--- ignore ---"! !


Error subclass: #NanoTraitCompositionException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTraitCompositionException commentStamp: '<historical>' prior: 0!
Signals invalid trait compositions.!



ClassDescription subclass: #NanoTraitDescription
	instanceVariableNames: 'users traitComposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTraitDescription commentStamp: 'ar 12/3/2009 23:42' prior: 0!
TraitDescription combines common behavior for both (instance) traits and (meta) class traits.!


!NanoTraitDescription methodsFor: 'operations' stamp: 'ar 12/20/2009 12:36'!
+ aTrait
	"Creates a composition with the receiver and aTrait"
	aTrait traitsDo:[:t| self == t ifTrue:[NanoTraitCompositionException
			signal: 'Trait ' , self asString, ' already in composition']].
	^NanoTraitComposition withAll: {self}, aTrait asTraitComposition! !

!NanoTraitDescription methodsFor: 'operations' stamp: 'ar 12/20/2009 12:37'!
- anArrayOfSelectors
	"Creates an exclusion"
	^NanoTraitExclusion
		with: self
		exclusions: anArrayOfSelectors! !

!NanoTraitDescription methodsFor: 'operations' stamp: 'ar 12/20/2009 12:35'!
@ anArrayOfAssociations 
	"Creates an alias"
	^ NanoTraitAlias with: self aliases: anArrayOfAssociations! !

!NanoTraitDescription methodsFor: 'operations' stamp: 'ar 12/19/2009 23:56'!
addSelectorSilently: selector withMethod: compiledMethod
	"Overridden to update the users of this trait"
	super addSelectorSilently: selector withMethod: compiledMethod.
	self users do:[:each| each updateTraits].! !

!NanoTraitDescription methodsFor: 'operations' stamp: 'ar 12/19/2009 23:54'!
installTraitsFrom: aTraitComposition
	super installTraitsFrom: aTraitComposition.
	self users do:[:each| each updateTraits].! !

!NanoTraitDescription methodsFor: 'operations' stamp: 'ar 12/20/2009 00:52'!
notifyOfRecategorizedSelector: element from: oldCategory to: newCategory
	super notifyOfRecategorizedSelector: element from: oldCategory to: newCategory.
	self users do:[:each| each classify: element under: newCategory from: oldCategory trait: self].! !

!NanoTraitDescription methodsFor: 'operations' stamp: 'ar 11/28/2007 20:59'!
traitsDo: aBlock
	aBlock value: self.! !


!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 12/20/2009 00:42'!
addTraitUser: aTrait
	users := self users copyWith: aTrait.
! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 12/20/2009 14:01'!
addUser: aTrait
	^self addTraitUser: aTrait! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 11/28/2007 20:57'!
allClassVarNames
	"Traits have no class var names"
	^#()! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 11/28/2007 20:58'!
classPool
	"Traits have no class pool"
	^ Dictionary new! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 12/20/2009 00:39'!
removeTraitUser: aTrait
	users := self users copyWithout: aTrait.
! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 12/20/2009 14:01'!
removeUser: aTrait
	^self removeTraitUser: aTrait! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 11/28/2007 20:58'!
sharedPools
	"Traits have no shared pools"
	^ Dictionary new! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 12/21/2009 15:13'!
trait
	^self! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 12/20/2009 13:14'!
traitComposition
	^traitComposition ifNil:[traitComposition := NanoTraitComposition new]
! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 12/20/2009 13:42'!
traitComposition: aTraitComposition
	traitComposition := aTraitComposition.
! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 11/28/2007 20:55'!
users
	^users ifNil:[#()]! !

!NanoTraitDescription methodsFor: 'accessing' stamp: 'ar 11/28/2007 20:55'!
users: aCollection
	users := aCollection! !


!NanoTraitDescription methodsFor: 'converting' stamp: 'ar 12/20/2009 12:35'!
asTraitComposition
	^NanoTraitComposition with: self! !


!NanoTraitDescription methodsFor: 'copying' stamp: 'ar 11/28/2007 20:55'!
copy 
	self error: 'Traits cannot be trivially copied'! !

!NanoTraitDescription methodsFor: 'copying' stamp: 'ar 11/28/2007 20:55'!
copyTraitExpression
	"Copy all except the actual traits"
	^self! !


!NanoTraitDescription methodsFor: 'fileIn/Out' stamp: 'ar 12/19/2009 16:09'!
fileOut
	"Create a file whose name is the name of the receiver with '.st' as the 
	extension, and file a description of the receiver onto it."
	^ self fileOutAsHtml: false! !

!NanoTraitDescription methodsFor: 'fileIn/Out' stamp: 'ar 12/19/2009 16:08'!
fileOutAsHtml: useHtml
	"File a description of the receiver onto a new file whose base name is the name of the receiver."

	| internalStream |
	internalStream := WriteStream on: (String new: 100).
	internalStream header; timeStamp.

	self fileOutOn: internalStream moveSource: false toFile: 0.
	internalStream trailer.

	FileStream writeSourceCodeFrom: internalStream baseName: self name isSt: true useHtml: useHtml.
! !


!NanoTraitDescription methodsFor: 'testing' stamp: 'ar 12/20/2009 02:35'!
includesTrait: aTrait
	^self == aTrait or:[super includesTrait: aTrait]! !

!NanoTraitDescription methodsFor: 'testing' stamp: 'ar 11/28/2007 21:01'!
isBaseTrait
	^false! !

!NanoTraitDescription methodsFor: 'testing' stamp: 'ar 11/28/2007 21:01'!
isClassTrait
	^false! !

!NanoTraitDescription methodsFor: 'testing' stamp: 'ar 11/28/2007 20:56'!
isTrait
	^true! !

!NanoTraitDescription methodsFor: 'testing' stamp: 'ar 11/28/2007 20:56'!
isTraitTransformation
	"Polymorphic with TraitTransformation"
	^false! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NanoTraitDescription class
	instanceVariableNames: ''!

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:47'!
conflict
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1 with: arg2
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1 with: arg2 with: arg3
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1 with: arg2 with: arg3 with: arg4
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1 with: arg2 with: arg3 with: arg4 with: arg5
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 with: arg7
	"This method has a trait conflict"
	^self traitConflict! !

!NanoTraitDescription class methodsFor: 'conflict methods' stamp: 'ar 12/19/2009 22:48'!
conflict: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 with: arg7 with: arg8
	"This method has a trait conflict"
	^self traitConflict! !


NanoTraitDescription subclass: #NanoClassTrait
	instanceVariableNames: 'baseTrait'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoClassTrait commentStamp: '<historical>' prior: 0!
While every class has an associated metaclass, a trait can have an associated classtrait, an instance of me. To preserve metaclass compatibility, the associated classtrait (if there is one) is automatically applied to the metaclass, whenever a trait is applied to a class. Consequently, a trait with an associated classtrait can only be applied to classes, whereas a trait without a classtrait can be applied to both classes and metaclasses.!


!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 23:00'!
baseTrait
	^baseTrait! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 12/20/2009 13:08'!
classSide
	^self! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 12/20/2009 03:55'!
definition
	^String streamContents: [:stream |
		stream nextPutAll: self name.
		stream cr; tab; nextPutAll: 'uses: ';
				nextPutAll: self traitComposition asString.
	].! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 12/20/2009 03:55'!
definitionST80
	^String streamContents: [:stream |
		stream nextPutAll: self name.
		stream cr; tab; nextPutAll: 'uses: ';
				nextPutAll: self traitComposition asString.
	].! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 5/11/2008 01:51'!
instanceSide
	^self baseTrait! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 21:44'!
name
	^baseTrait name, ' classTrait'! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 21:45'!
new
	baseTrait ifNotNil:[self error: 'Already initialized'].
	baseTrait := self basicNew initialize.
	baseTrait
		superclass: nil 
		methodDictionary: MethodDictionary new 
		format: Object format.
	^baseTrait! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 12/19/2009 18:38'!
soleInstance
	^baseTrait! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 12/20/2009 02:55'!
theMetaClass
	^self! !

!NanoClassTrait methodsFor: 'accessing' stamp: 'ar 12/20/2009 14:29'!
theNonMetaClass
	"Sent to a class or metaclass, always return the class"
	^baseTrait! !


!NanoClassTrait methodsFor: 'testing' stamp: 'ar 11/18/2007 22:59'!
isClassTrait
	^true! !

!NanoClassTrait methodsFor: 'testing' stamp: 'ar 11/18/2007 22:21'!
isMeta
	^true! !


!NanoClassTrait methodsFor: 'initialize' stamp: 'ar 12/21/2009 15:14'!
updateTraitsFrom: instanceTraits
	"Update me from the given instance traits"
	| map newTraits trait |
	map := Dictionary new.
	self traitComposition do:[:composed| map at: composed trait put: composed].
	newTraits := (instanceTraits collect:[:composed|
		trait := composed trait classTrait.
		map at: trait ifAbsent:[trait]] 
	), (self traitComposition select:[:comp| comp trait isBaseTrait]).

	self installTraitsFrom: newTraits! !

!NanoClassTrait methodsFor: 'initialize' stamp: 'ar 12/20/2009 12:36'!
uses: aTraitComposition
	| newTraits |
	newTraits := (aTraitComposition isTrait or:[aTraitComposition isTraitTransformation]) 
		ifTrue:[NanoTraitComposition with: aTraitComposition]
		ifFalse:[(aTraitComposition isKindOf: SequenceableCollection)
					ifTrue:[NanoTraitComposition withAll: aTraitComposition asArray]
					ifFalse:[self error: 'Invalid traits specification']].
	newTraits traitsDo:[:t|
		(t isBaseTrait and:[t classSide hasMethods]) 
			ifTrue:[self error: 'Cannot add: ', t].
		(t isClassTrait and:[(baseTrait includesTrait: t baseTrait) not]) 
			ifTrue:[self error: 'Cannot add: ', t].
	].
	self installTraitsFrom: newTraits.! !


!NanoClassTrait methodsFor: 'compiling' stamp: 'ar 12/21/2009 20:18'!
bindingOf: varName
	"Answer the binding of some variable resolved in the scope of the receiver"
	^baseTrait bindingOf: varName! !


!NanoClassTrait methodsFor: 'monticello' stamp: 'ar 12/22/2009 08:49'!
asMCDefinition
	^Smalltalk at: #MCClassTraitDefinition ifPresent:[:aClass|
		aClass
			baseTraitName: self baseTrait name
			classTraitComposition: self traitCompositionString
	].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NanoClassTrait class
	instanceVariableNames: ''!

!NanoClassTrait class methodsFor: 'instance creation' stamp: 'ar 12/20/2009 12:34'!
new
	| newMeta |
	newMeta := super new.
	newMeta 
		superclass: NanoTrait 
		methodDictionary: MethodDictionary new 
		format: NanoTrait format.
	^newMeta! !


NanoTraitDescription subclass: #NanoTrait
	instanceVariableNames: 'name environment category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTrait commentStamp: '<historical>' prior: 0!
Each trait in the system is represented as an instance of me. Like Class, I concretize my superclass by providing instance variables for the name and the environment.!


!NanoTrait methodsFor: 'accessing' stamp: 'ar 12/21/2009 19:49'!
baseTrait
	^self! !

!NanoTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 15:37'!
category
	"Answer the system organization category for the receiver. First check whether the
	category name stored in the ivar is still correct and only if this fails look it up
	(latter is much more expensive)"

	| result |
	category ifNotNilDo: [ :symbol |
		((SystemOrganization listAtCategoryNamed: symbol) includes: self name)
			ifTrue: [ ^symbol ] ].
	category := (result := SystemOrganization categoryOfElement: self name).
	^result! !

!NanoTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 15:38'!
category: aString 
	"Categorize the receiver under the system category, aString, removing it from 
	any previous categorization."

	| oldCategory |
	oldCategory := category.
	aString isString
		ifTrue: [
			category := aString asSymbol.
			SystemOrganization classify: self name under: category ]
		ifFalse: [self errorCategoryName].
	SystemChangeNotifier uniqueInstance
		class: self recategorizedFrom: oldCategory to: category! !

!NanoTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 21:48'!
classTrait
	^self class! !

!NanoTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 15:37'!
environment
	^environment! !

!NanoTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 15:37'!
environment: anObject
	environment := anObject! !

!NanoTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 15:37'!
name
	^name! !

!NanoTrait methodsFor: 'accessing' stamp: 'ar 11/18/2007 15:37'!
name: aSymbol
	name := aSymbol! !


!NanoTrait methodsFor: 'initialize' stamp: 'ar 12/21/2009 20:14'!
definition
	^String streamContents: [:stream |
		stream nextPutAll: 'NanoTrait named: ';
				store: self name.
		stream cr; tab; nextPutAll: 'uses: ';
				nextPutAll: self traitComposition asString.
		stream cr; tab; nextPutAll: 'category: ';
				store: self category asString].! !

!NanoTrait methodsFor: 'initialize' stamp: 'ar 12/20/2009 03:55'!
definitionST80
	^String streamContents: [:stream |
		stream nextPutAll: 'Trait named: ';
				store: self name.
		stream cr; tab; nextPutAll: 'uses: ';
				nextPutAll: self traitComposition asString.
		stream cr; tab; nextPutAll: 'category: ';
				store: self category asString].! !

!NanoTrait methodsFor: 'initialize' stamp: 'ar 11/18/2007 15:33'!
isValidTraitName: aSymbol
	^(aSymbol isEmptyOrNil
		or: [aSymbol first isLetter not]
		or: [aSymbol anySatisfy: [:character | character isAlphaNumeric not]]) not! !

!NanoTrait methodsFor: 'initialize' stamp: 'ar 11/28/2007 21:00'!
obsolete
	self name: ('AnObsolete' , self name) asSymbol.
	super obsolete! !

!NanoTrait methodsFor: 'initialize' stamp: 'ar 11/28/2007 21:00'!
removeFromSystem
	self removeFromSystem: true! !

!NanoTrait methodsFor: 'initialize' stamp: 'ar 11/28/2007 21:00'!
removeFromSystem: logged
	self environment forgetClass: self logged: logged.
	self obsolete! !

!NanoTrait methodsFor: 'initialize' stamp: 'ar 11/18/2007 15:41'!
rename: aString 
	"The new name of the receiver is the argument, aString."

	| newName |
	(newName := aString asSymbol) ~= self name
		ifFalse: [^ self].
	(self environment includesKey: newName)
		ifTrue: [^ self error: newName , ' already exists'].
	(Undeclared includesKey: newName)
		ifTrue: [self inform: 'There are references to, ' , aString printString , '
from Undeclared. Check them after this change.'].
	self environment renameClass: self as: newName.
	name := newName! !

!NanoTrait methodsFor: 'initialize' stamp: 'ar 11/18/2007 15:33'!
setName: aSymbol andRegisterInCategory: categorySymbol environment: aSystemDictionary
	(self isValidTraitName: aSymbol) ifFalse: [self error:'Invalid trait name'].

	(self environment == aSystemDictionary
		and: [self name = aSymbol
			and: [self category = categorySymbol]]) ifTrue: [^self].
		
	((aSystemDictionary includes: aSymbol) and: [(aSystemDictionary at: aSymbol) ~~ self])
		ifTrue: [self error: 'The name ''' , aSymbol , ''' is already used'].

	(self environment notNil and: [self name notNil and: [self name ~= aSymbol]]) ifTrue: [
		self environment renameClass: self as: aSymbol].
	
	self name: aSymbol.
	self environment: aSystemDictionary.	
	self environment at: self name put: self.
	self environment organization classify: self name under: categorySymbol.
	^ true! !


!NanoTrait methodsFor: 'fileIn/Out' stamp: 'ar 12/19/2009 18:06'!
fileOutOn: aFileStream moveSource: moveSource toFile: fileIndex
	super fileOutOn: aFileStream moveSource: moveSource toFile: fileIndex.
	self classSide hasMethods ifTrue:[
		aFileStream cr; nextPutAll: '"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!!'; cr; cr.
			self classSide
				fileOutOn: aFileStream
				moveSource: moveSource
				toFile: fileIndex].! !


!NanoTrait methodsFor: 'testing' stamp: 'ar 5/11/2008 01:57'!
hasClassTrait
	^true! !

!NanoTrait methodsFor: 'testing' stamp: 'ar 11/18/2007 22:59'!
isBaseTrait
	^true! !

!NanoTrait methodsFor: 'testing' stamp: 'ar 11/18/2007 15:41'!
isObsolete
	"Return true if the receiver is obsolete."
	^(self environment at: name ifAbsent: [nil]) ~~ self! !


!NanoTrait methodsFor: 'compiling' stamp: 'ar 12/21/2009 20:17'!
bindingOf: varName
	"Answer the binding of some variable resolved in the scope of the receiver"
	^self environment bindingOf: varName asSymbol.! !


!NanoTrait methodsFor: 'monticello' stamp: 'ar 12/22/2009 08:48'!
asClassDefinition
	^Smalltalk at: #MCTraitDefinition ifPresent:[:aClass|
		aClass
			name: self name
			traitComposition: self traitCompositionString
			category: self category 
			comment: self organization classComment asString
			commentStamp: self organization commentStamp].! !

!NanoTrait methodsFor: 'monticello' stamp: 'ar 12/22/2009 08:46'!
classDefinitions
	| definitions |
	definitions := OrderedCollection with: self asClassDefinition.
	(self hasClassTrait
		and: [self classTrait hasTraitComposition]
		and: [self classTrait traitComposition isEmpty not])
			ifTrue: [definitions add: self classTrait asMCDefinition].
	^definitions asArray! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NanoTrait class
	instanceVariableNames: ''!

!NanoTrait class methodsFor: 'installing' stamp: 'ar 12/21/2009 19:17'!
install		"NanoTrait install"
	"Installs NanoTraits"
	Trait traitOverride: self. 				"Create all new traits as NanoTraits"
	self updateTraits: Smalltalk allTraits.	"And convert everything to NanoTraits"
! !

!NanoTrait class methodsFor: 'installing' stamp: 'ar 12/22/2009 10:06'!
updateTraits: aCollection
	"Convert all the traits in aCollection to NanoTraits. Used during installation."
	"Trait traitOverride: NanoTrait.
	NanoTrait updateTraits:{
		TSequencedStreamTest. TGettableStreamTest. TReadStreamTest.
		TStreamTest. TPuttableStreamTest. TWriteStreamTest
	}"
	| remain processed classes oldTrait classDef newTrait count instTraits classSelectors |
	Trait traitOverride == self ifFalse:[self error: 'What are you doing???'].
	remain := (aCollection reject:[:tc| tc isKindOf: self]) asSet.
	processed := Set new.
	classes := Set new.
	count := 0.
	'Converting ....' displayProgressAt: Sensor cursorPoint from: 1 to: remain size during:[:bar|
	[remain isEmpty] whileFalse:[
		"Pick any trait whose traits are already converted"
		oldTrait := remain detect:[:any| 
			any traitComposition traits allSatisfy:[:t| (Smalltalk at: t name) isKindOf: self].
		] ifNone:[self error: 'Cannot convert cyclic traits'].
		remain remove: oldTrait.

		bar value: (count := count +1).
		ProgressNotification signal: '' extra: 'Converting ', oldTrait name.

		"Silently remove the old trait class and recreate it based on NanoTrait"
		classDef := oldTrait definition.
		Smalltalk removeKey: oldTrait name.

		"Create the NanoTrait from the same definition"
		newTrait := Compiler evaluate: classDef.

		"Copy local methods to new trait"
		oldTrait localSelectors do:[:sel|
			newTrait 
				compile: (oldTrait sourceCodeAt: sel)
				classified: (oldTrait organization categoryOfElement: sel)].
		oldTrait classSide localSelectors do:[:sel|
			newTrait classSide
				compile: (oldTrait classSide sourceCodeAt: sel)
				classified: (oldTrait classSide organization categoryOfElement: sel)].

		newTrait selectors sort = oldTrait selectors sort
			ifFalse:[self error: 'Something went VERY wrong'].
		newTrait classSide selectors sort = oldTrait classSide selectors sort
			ifFalse:[self error: 'Something went VERY wrong'].

		processed add: oldTrait.
		classes addAll: (oldTrait users reject:[:aClass| aClass isObsolete]).
	].
	].

	classes := classes asArray select:[:cls| cls isKindOf: ClassDescription].
	'Updating ....' displayProgressAt: Sensor cursorPoint from: 1 to: classes size during:[:bar|
	"The traits are all converted, next update the classes"
	classes keysAndValuesDo:[:index :aClass|
		bar value: index.
		ProgressNotification signal: '' extra: 'Updating ', aClass name.

		instTraits := Compiler evaluate: aClass traitComposition asString.
		"Keep the local selectors from before"
		localSelectors := aClass localSelectors.
		classSelectors := aClass class localSelectors.
		"Nuke the old traits composition"
		aClass traitComposition: nil.
		aClass class traitComposition: nil.
		"Install the new one"
		aClass uses: instTraits.
		aClass == Behavior ifFalse:[
			"Remove the old trait (now local) selectors"
			(aClass selectors reject:[:sel| localSelectors includes: sel]) do:[:sel| 
				aClass removeSelectorSilently: sel.
				(aClass includesSelector: sel) ifFalse:[self halt: 'Where is the code?'].
			].
			(aClass class selectors reject:[:sel| classSelectors includes: sel]) do:[:sel|
				aClass class removeSelectorSilently: sel.
				(aClass class includesSelector: sel) ifFalse:[self halt: 'Where is the code?'].
			].
		].
	].
	].

	"Finally, obsolete all the old traits"
	processed do:[:trait| trait obsolete].
! !


!NanoTrait class methodsFor: 'instance creation' stamp: 'ar 5/1/2008 21:16'!
named: aSymbol uses: aTraitCompositionOrCollection category: aString
	| env |
	env := self environment.
	^self
		named: aSymbol
		uses: aTraitCompositionOrCollection
		category: aString
		env: env! !

!NanoTrait class methodsFor: 'instance creation' stamp: 'ar 12/20/2009 12:34'!
named: aSymbol uses: aTraitComposition category: aString env: anEnvironment
	| trait oldTrait systemCategory oldCategory |
	systemCategory := aString asSymbol.
	oldTrait := anEnvironment at: aSymbol ifAbsent: [nil].
	oldTrait ifNil:[
		trait := NanoClassTrait new new.
	] ifNotNil:[
		oldCategory := oldTrait category.
		trait := oldTrait.
	].
	(trait isKindOf: NanoTrait) ifFalse: [
		^self error: trait name , ' is not a Trait'].
	trait
		setName: aSymbol
		andRegisterInCategory: systemCategory
		environment: anEnvironment.

	trait uses: aTraitComposition.
	
	"... notify interested clients ..."
	oldTrait ifNil:[
		SystemChangeNotifier uniqueInstance classAdded: trait inCategory: systemCategory.
	] ifNotNil:[
		systemCategory = oldCategory  ifFalse:[
			SystemChangeNotifier uniqueInstance class: trait 
				recategorizedFrom: oldTrait category to: systemCategory].
	].
	^ trait! !


!NanoTrait class methodsFor: 'accessing' stamp: 'ar 12/21/2009 20:24'!
newTraitComposition
	"Creates a new TraitComposition"
	^NanoTraitComposition new! !


AdditionalMethodState variableSubclass: #NanoTraitMethodState
	instanceVariableNames: 'originalTraitMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTraitMethodState commentStamp: '<historical>' prior: 0!
Additional method state for trait provided methods.!


!NanoTraitMethodState methodsFor: 'accessing' stamp: 'ar 12/22/2009 07:54'!
methodHome
	"The behavior (trait/class) this method was originally defined in. 
	Derived from the originalTraitMethod if any."
	^originalTraitMethod ifNil:[super methodHome] ifNotNil:[:m| m methodHome]! !

!NanoTraitMethodState methodsFor: 'accessing' stamp: 'ar 12/22/2009 07:44'!
originalTraitMethod
	"The original method from the trait"
	^originalTraitMethod! !

!NanoTraitMethodState methodsFor: 'accessing' stamp: 'ar 12/22/2009 07:44'!
originalTraitMethod: aCompiledMethod
	"The original method from the trait"
	originalTraitMethod := aCompiledMethod! !


Object subclass: #NanoTraitTransformation
	instanceVariableNames: 'subject users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTraitTransformation commentStamp: '<historical>' prior: 0!
A trait transformation is an instance of one of my concrete subclasses, TraitAlias or TraitExclusion. These represent a transformation of a trait, specified by the alias and exclusion operators. 

I define an instance variable named subject which holds the object that is transformed.  Thus, an alias transformation has as its subject a trait, and a trait exclusion has as its subject either a trait alias or a trait. Each of the concrete transformation classes implement the method allSelectors according to the transformation it represents. 
!


!NanoTraitTransformation methodsFor: 'converting' stamp: 'ar 12/20/2009 12:36'!
+ aTrait
	"Just like ordered collection"
	^NanoTraitComposition withAll: {self. aTrait}! !

!NanoTraitTransformation methodsFor: 'converting' stamp: 'ar 12/7/2009 20:39'!
- anArrayOfSelectors
	^self subclassResponsibility! !

!NanoTraitTransformation methodsFor: 'converting' stamp: 'ar 12/7/2009 20:39'!
@ anArrayOfAssociations
	^self subclassResponsibility! !

!NanoTraitTransformation methodsFor: 'converting' stamp: 'ar 12/20/2009 13:44'!
asTraitComposition
	^NanoTraitComposition with: self! !

!NanoTraitTransformation methodsFor: 'converting' stamp: 'ar 11/18/2007 17:44'!
asTraitTransform
	^self! !


!NanoTraitTransformation methodsFor: 'accessing' stamp: 'ar 12/20/2009 00:42'!
addTraitUser: aTrait
	users := users copyWith: aTrait.
	subject addTraitUser: aTrait.
! !

!NanoTraitTransformation methodsFor: 'accessing' stamp: 'ar 11/18/2007 19:12'!
allTraits
	^subject allTraits! !

!NanoTraitTransformation methodsFor: 'accessing' stamp: 'ar 12/20/2009 00:40'!
removeTraitUser: aTrait
	users := users copyWithout: aTrait.
	subject removeTraitUser: aTrait.! !

!NanoTraitTransformation methodsFor: 'accessing' stamp: 'ar 11/18/2007 19:12'!
subject: aSubject
	subject := aSubject.! !

!NanoTraitTransformation methodsFor: 'accessing' stamp: 'ar 12/21/2009 15:13'!
trait
	^subject trait! !

!NanoTraitTransformation methodsFor: 'accessing' stamp: 'ar 11/22/2007 06:08'!
traitsDo: aBlock
	^subject traitsDo: aBlock! !


!NanoTraitTransformation methodsFor: 'operations' stamp: 'ar 11/22/2007 06:18'!
copyTraitExpression
	"Copy all except the actual traits"
	^self subclassResponsibility! !

!NanoTraitTransformation methodsFor: 'operations' stamp: 'ar 11/22/2007 06:12'!
selectorsAndMethodsDo: aBlock
	"enumerates all selectors and methods in a trait composition"
	^self subclassResponsibility! !

!NanoTraitTransformation methodsFor: 'operations' stamp: 'ar 11/22/2007 06:12'!
updateSelector: aSelector withTraitMethod: compiledMethod from: aTrait
	"broadcasts the change of a selector to all users of a trait"
	^self subclassResponsibility! !

!NanoTraitTransformation methodsFor: 'operations' stamp: 'ar 12/20/2009 00:13'!
updateTraits
	users do:[:each| each updateTraits].! !


!NanoTraitTransformation methodsFor: 'testing' stamp: 'ar 12/20/2009 02:35'!
includesTrait: aTrait
	^subject includesTrait: aTrait! !

!NanoTraitTransformation methodsFor: 'testing' stamp: 'ar 5/11/2008 02:13'!
isAliasSelector: selector
	^subject isAliasSelector: selector! !

!NanoTraitTransformation methodsFor: 'testing' stamp: 'ar 5/11/2008 02:17'!
isLocalAliasSelector: selector
	^false! !

!NanoTraitTransformation methodsFor: 'testing' stamp: 'ar 11/22/2007 04:24'!
isTraitTransformation
	"Polymorphic with Trait"
	^true! !


!NanoTraitTransformation methodsFor: 'initialize' stamp: 'ar 11/18/2007 17:32'!
initialize
	super initialize.
	users := #().! !


NanoTraitTransformation subclass: #NanoTraitAlias
	instanceVariableNames: 'aliases'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTraitAlias commentStamp: '<historical>' prior: 0!
A trait transformation representing the alias (->) operator.!


!NanoTraitAlias methodsFor: 'converting' stamp: 'ar 12/20/2009 12:37'!
- anArrayOfSelectors
	^NanoTraitExclusion
		with: self
		exclusions: anArrayOfSelectors! !

!NanoTraitAlias methodsFor: 'converting' stamp: 'ar 12/20/2009 12:35'!
@ anArrayOfAssociations 
	^NanoTraitAlias 
		with: subject
		aliases: (anArrayOfAssociations, self aliases)! !


!NanoTraitAlias methodsFor: 'accessing' stamp: 'ar 12/19/2009 21:55'!
aliases
	"Collection of associations where key is the
	alias and value the original selector."
	^aliases! !

!NanoTraitAlias methodsFor: 'accessing' stamp: 'ar 12/19/2009 21:56'!
aliases: aCollection
	"Collection of associations where key is the
	alias and value the original selector."
	aliases := aCollection! !


!NanoTraitAlias methodsFor: 'operations' stamp: 'ar 12/20/2009 12:35'!
copyTraitExpression
	"Copy all except the actual traits"
	^NanoTraitAlias 
		with: subject
		aliases: aliases! !

!NanoTraitAlias methodsFor: 'operations' stamp: 'ar 12/19/2009 22:00'!
includesSelector: selector
	"Answers true if the receiver provides the selector"
	^(subject includesSelector: selector) or:[aliases anySatisfy:[:assoc| assoc key == selector]]! !

!NanoTraitAlias methodsFor: 'operations' stamp: 'ar 12/20/2009 13:49'!
printOn: s
	"Answer the trait composition string (used for class definitions)"
	s nextPutAll: subject asString.
	s nextPutAll: ' @ {'.
	aliases do:[:assoc| s print: assoc] separatedBy:[s nextPutAll:'. '].
	s nextPutAll: '}'.
! !

!NanoTraitAlias methodsFor: 'operations' stamp: 'ar 12/19/2009 22:14'!
selectorsAndMethodsDo: aBlock
	"enumerates all selectors and methods in a trait composition"
	subject selectorsAndMethodsDo:[:sel :meth|
		aBlock value: sel value: meth.
	].
	aliases do:[:assoc| | method |
		"Method can be nil during removals"
		method := subject compiledMethodAt: assoc value ifAbsent:[nil].
		method ifNotNil:[aBlock value: assoc key value: method].
	].! !


!NanoTraitAlias methodsFor: 'initialize-release' stamp: 'ar 12/19/2009 22:00'!
initialize
	super initialize.
	aliases := #().! !

!NanoTraitAlias methodsFor: 'initialize-release' stamp: 'ar 12/20/2009 12:37'!
initializeFrom: anArrayOfAssociations
	| newNames |
	newNames := (anArrayOfAssociations collect: [:each | each key]) asIdentitySet.
	newNames size < anArrayOfAssociations size ifTrue: [
		NanoTraitCompositionException signal: 'Cannot use the same alias name twice'].
	anArrayOfAssociations do: [:each |
		(newNames includes: each value) ifTrue: [
			NanoTraitCompositionException signal: 'Cannot define an alias for an alias']].
	aliases := anArrayOfAssociations.
! !


!NanoTraitAlias methodsFor: 'testing' stamp: 'ar 12/19/2009 22:01'!
isAliasSelector: selector
	^(self isLocalAliasSelector: selector) or:[super isAliasSelector: selector]! !

!NanoTraitAlias methodsFor: 'testing' stamp: 'ar 12/19/2009 22:01'!
isLocalAliasSelector: selector
	^(aliases anySatisfy:[:assoc| assoc key == selector])! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NanoTraitAlias class
	instanceVariableNames: ''!

!NanoTraitAlias class methodsFor: 'instance creation' stamp: 'ar 12/20/2009 12:36'!
assertValidAliasDefinition: anArrayOfAssociations
	"Throw an exceptions if the alias definition is not valid.
	It is expected to be a collection of associations and
	the number of arguments of the alias selector has to
	be the same as the original selector."

	((anArrayOfAssociations isKindOf: Collection) and: [
		anArrayOfAssociations allSatisfy: [:each |
			each isKindOf: Association]]) ifFalse: [
		self error: 'Invalid alias definition: Not a collection of associations.'].
	
	(anArrayOfAssociations allSatisfy: [:association |
		(association key numArgs = association value numArgs and: [
			(association key numArgs = -1) not])]) ifFalse: [
		NanoTraitCompositionException signal: 'Invalid alias definition: Alias and original selector have to have the same number of arguments.']! !

!NanoTraitAlias class methodsFor: 'instance creation' stamp: 'ar 11/18/2007 17:45'!
with: aTraitComposition aliases: anArrayOfAssociations
	self assertValidAliasDefinition: anArrayOfAssociations.
	^self new
		subject: aTraitComposition;
		initializeFrom: anArrayOfAssociations;
		yourself! !


NanoTraitTransformation subclass: #NanoTraitExclusion
	instanceVariableNames: 'exclusions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NanoTraits-Kernel'!
!NanoTraitExclusion commentStamp: '<historical>' prior: 0!
A trait transformation representing the exclusion (-) operator.!


!NanoTraitExclusion methodsFor: 'converting' stamp: 'ar 12/20/2009 12:38'!
- anArrayOfSelectors
	^NanoTraitExclusion
		with: subject
		exclusions: (anArrayOfSelectors, exclusions asArray)! !

!NanoTraitExclusion methodsFor: 'converting' stamp: 'ar 12/20/2009 12:37'!
@ anArrayOfAssociations 

	NanoTraitCompositionException signal: 'Invalid trait exclusion. Aliases have to be specified before exclusions.'
! !


!NanoTraitExclusion methodsFor: 'initialize' stamp: 'ar 11/18/2007 17:16'!
initialize
	super initialize.
	exclusions := Set new.
! !


!NanoTraitExclusion methodsFor: 'composition' stamp: 'ar 12/20/2009 12:38'!
copyTraitExpression
	"Copy all except the actual traits"
	^NanoTraitExclusion 
		with: subject
		exclusions: exclusions asArray! !

!NanoTraitExclusion methodsFor: 'composition' stamp: 'ar 11/27/2007 22:10'!
includesSelector: selector
	"Answers true if the receiver provides the selector"
	^(subject includesSelector: selector) and:[(exclusions includes: selector) not]! !

!NanoTraitExclusion methodsFor: 'composition' stamp: 'ar 12/20/2009 13:48'!
printOn: aStream
	"Answer the trait composition string (used for class definitions)"
	aStream nextPutAll: subject asString.
	aStream nextPutAll: ' - {'.
	exclusions asArray sort do:[:exc| aStream store: exc] separatedBy:[aStream nextPutAll: '. '].
	aStream nextPutAll: '}'.! !

!NanoTraitExclusion methodsFor: 'composition' stamp: 'ar 11/22/2007 06:12'!
selectorsAndMethodsDo: aBlock
	"enumerates all selectors and methods in a trait composition"
	^subject selectorsAndMethodsDo:[:sel :meth|
		(exclusions includes: sel) ifFalse:[aBlock value: sel value: meth].
	].! !


!NanoTraitExclusion methodsFor: 'accessing' stamp: 'ar 11/18/2007 18:04'!
exclusions
	^exclusions! !

!NanoTraitExclusion methodsFor: 'accessing' stamp: 'ar 11/18/2007 18:04'!
exclusions: aCollection
	exclusions := Set withAll: aCollection! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NanoTraitExclusion class
	instanceVariableNames: ''!

!NanoTraitExclusion class methodsFor: 'instance creation' stamp: 'ar 11/18/2007 18:04'!
with: aTraitComposition exclusions: anArrayOfSelectors
	^self new
		subject: aTraitComposition;
		exclusions: anArrayOfSelectors;
		yourself
! !
